@@ -1,5 +1,5 @@
 /* tc-mep.h -- Header file for tc-mep.c.
-   Copyright (C) 2001-2017 Free Software Foundation, Inc.
+   Copyright (C) 2001-2021 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -53,7 +53,6 @@ extern void mep_apply_fix (struct fix *, valueT *, segT);
 
 /* Call md_pcrel_from_section(), not md_pcrel_from().  */
 #define MD_PCREL_FROM_SECTION(FIXP, SEC) md_pcrel_from_section (FIXP, SEC)
-extern long md_pcrel_from_section (struct fix *, segT);
 
 #define tc_frob_file() mep_frob_file ()
 extern void mep_frob_file (void);
@@ -115,6 +114,6 @@ extern flagword mep_elf_section_flags  (flagword, bfd_vma, int);
 
 /* The values of the following enum are for use with parinsnum, which
    is a variable in md_assemble that keeps track of whether or not the
-   next instruction is expected to be the first or second instrucion in
+   next instruction is expected to be the first or second instruction in
    a parallelization group.  */
 typedef enum exp_par_insn_{FIRST, SECOND} EXP_PAR_INSN;
