@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.  */
+/* A Bison parser, made by GNU Bison 3.5.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2013 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,6 +31,9 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
 #ifndef YY_YY_DEFFILEP_H_INCLUDED
 # define YY_YY_DEFFILEP_H_INCLUDED
 /* Debug traces.  */
@@ -66,7 +70,7 @@ extern int yydebug;
     READ = 276,
     WRITE = 277,
     EXECUTE = 278,
-    SHARED = 279,
+    SHARED_K = 279,
     NONAMEU = 280,
     NONAMEL = 281,
     DIRECTIVE = 282,
@@ -97,7 +101,7 @@ extern int yydebug;
 #define READ 276
 #define WRITE 277
 #define EXECUTE 278
-#define SHARED 279
+#define SHARED_K 279
 #define NONAMEU 280
 #define NONAMEL 281
 #define DIRECTIVE 282
@@ -107,10 +111,9 @@ extern int yydebug;
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
-typedef union YYSTYPE YYSTYPE;
 union YYSTYPE
 {
-#line 113 "deffilep.y" /* yacc.c:1915  */
+#line 114 "deffilep.y"
 
   char *id;
   const char *id_const;
@@ -118,8 +121,10 @@ union YYSTYPE
   bfd_vma vma;
   char *digits;
 
-#line 122 "deffilep.h" /* yacc.c:1915  */
+#line 125 "deffilep.h"
+
 };
+typedef union YYSTYPE YYSTYPE;
 # define YYSTYPE_IS_TRIVIAL 1
 # define YYSTYPE_IS_DECLARED 1
 #endif
